@@ -515,6 +515,75 @@ def _ipmitool_timing_args():
     ]
 
 
+def choose_cipher_suite(actual_ciper_suite):
+    """Gives the possible next avaible cipher suite version.
+
+    Based on CONF.ipmi.cipher_suite_versions and the last cipher suite version
+    used that failed. This function is only called if the node doesn't have
+    cipher_suite set. Starts using the last element of the list and decreasing
+    the index.
+
+    :param actual_ciper_suite: latest cipher suite used in the
+        ipmi call.
+
+    :returns: the next possible cipher suite or None in case of empty
+        configuration.
+    """
+    available_cs_versions = CONF.ipmi.cipher_suite_versions
+    if not available_cs_versions:
+        return None
+
+    if actual_ciper_suite is None:
+        return available_cs_versions[-1]
+    else:
+        try:
+            cs_index = available_cs_versions.index(actual_ciper_suite)
+        except ValueError:
+            return available_cs_versions[-1]
+
+        return available_cs_versions[max(cs_index - 1, 0)]
+
+
+def check_cipher_suite_errors(cmd_stderr):
+    """Checks if the command stderr contains cipher suite errors.
+
+    :param cmd_stderr: The command stderr.
+
+    :returns: True if the cmd_stderr contains a cipher suite error,
+        False otherwise.
+    """
+    cs_errors = ["Unsupported cipher suite ID",
+                 "Error in open session response message :"
+                 " no matching cipher suite"]
+    for cs_err in cs_errors:
+        if cmd_stderr is not None and cs_err in cmd_stderr:
+            return True
+    return False
+
+
+def update_cipher_suite_cmd(actual_cs, args):
+    """Updates variables and the cipher suite cmd.
+
+    This function updates the values for all parameters so they
+    can be used in the next retry of _exec_ipmitool.
+
+    :param actual_cs: a string that represents the cipher suite that was
+        used in the command.
+    :param args: a list that contains the ipmitool command that was executed.
+
+    :returns: a tuple with the new values (actual_cs, args)
+    """
+    actual_cs = choose_cipher_suite(actual_cs)
+    if '-C' in args:
+        cs_index = args.index('-C') + 1
+        args[cs_index] = actual_cs
+    else:
+        args.append('-C')
+        args.append(actual_cs)
+
+    return (actual_cs, args)
+
+
 def _exec_ipmitool(driver_info, command, check_exit_code=None,
                    kill_on_timeout=False):
     """Execute the ipmitool command.
@@ -533,6 +602,10 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None,
     """
     args = _get_ipmitool_args(driver_info)
 
+    change_cs = (CONF.ipmi.cipher_suite_versions != []
+                 and driver_info.get('cipher_suite') is None)
+    actual_cs = None
+
     timeout = CONF.ipmi.command_retry_timeout
 
     args.extend(_ipmitool_timing_args())
@@ -570,6 +643,11 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None,
                 out, err = utils.execute(*cmd_args, **extra_args)
                 return out, err
             except processutils.ProcessExecutionError as e:
+                if change_cs and check_cipher_suite_errors(e.stderr):
+                    actual_cs, args = update_cipher_suite_cmd(
+                        actual_cs, args)
+                else:
+                    change_cs = False
                 with excutils.save_and_reraise_exception() as ctxt:
                     err_list = [
                         x for x in (
@@ -939,25 +1017,6 @@ class IPMIPower(base.PowerInterface):
             call).
 
         """
-        # NOTE(TheJulia): Temporary until we promote detect_vendor as
-        # a higher level management method and able to automatically
-        # run detection upon either power state sync or in the enrollment
-        # to management step.
-        try:
-            properties = task.node.properties
-            if not properties.get('vendor'):
-                # We have no vendor stored, so we'll go ahead and
-                # call to store it.
-                vendor = task.driver.management.detect_vendor(task)
-                if vendor:
-                    task.upgrade_lock()
-                    props = task.node.properties
-                    props['vendor'] = vendor
-                    task.node.properties = props
-                    task.node.save()
-        except exception.UnsupportedDriverExtension:
-            pass
-
         driver_info = _parse_driver_info(task.node)
         return _power_status(driver_info)
 
@@ -1244,12 +1303,7 @@ class IPMIManagement(base.ManagementInterface):
 
     @METRICS.timer('IPMIManagement.detect_vendor')
     def detect_vendor(self, task):
-        """Detects, stores, and returns the hardware vendor.
-
-        If the Node object ``properties`` field does not already contain
-        a ``vendor`` field, then this method is intended to query
-        Detects the BMC hardware vendor and stores the returned value
-        with-in the Node object ``properties`` field if detected.
+        """Detects and returns the hardware vendor.
 
         :param task: A task from TaskManager.
         :raises: InvalidParameterValue if an invalid component, indicator
@@ -1258,20 +1312,11 @@ class IPMIManagement(base.ManagementInterface):
         :returns: String representing the BMC reported Vendor or
                   Manufacturer, otherwise returns None.
         """
-        try:
-            driver_info = _parse_driver_info(task.node)
-            out, err = _exec_ipmitool(driver_info, "mc info")
-            re_obj = re.search("Manufacturer Name .*: (.+)", out)
-            if re_obj:
-                bmc_vendor = str(re_obj.groups('')[0]).lower().split(':')
-                # Pull unparsed data and save the vendor
-                return bmc_vendor[-1]
-        except (exception.PasswordFileFailedToCreate,
-                processutils.ProcessExecutionError) as e:
-            LOG.warning('IPMI get boot device failed to detect vendor '
-                        'of bmc for %(node)s. Error %(err)s',
-                        {'node': task.node.uuid,
-                         'err': e})
+        driver_info = _parse_driver_info(task.node)
+        out, err = _exec_ipmitool(driver_info, "mc info")
+        re_obj = re.search("Manufacturer Name *: (.+)", out)
+        if re_obj:
+            return re_obj.group(1).lower()
 
     @METRICS.timer('IPMIManagement.get_sensors_data')
     def get_sensors_data(self, task):
