@@ -239,7 +239,12 @@ class RedfishManagement(base.ManagementInterface):
 
         # Ensure that boot mode is synced with what is set.
         # Some BMCs reset it to default (BIOS) when changing the boot device.
-        boot_mode_utils.sync_boot_mode(task)
+        # It should only be synced on these vendors as other vendor
+        # implementations will result in an error
+        # (see https://storyboard.openstack.org/#!/story/2008712)
+        vendor = task.node.properties.get('vendor', None)
+        if vendor and vendor.lower() == 'supermicro':
+            boot_mode_utils.sync_boot_mode(task)
 
     def get_boot_device(self, task):
         """Get the current boot device for a node.
@@ -303,6 +308,19 @@ class RedfishManagement(base.ManagementInterface):
                          {'node': task.node.uuid, 'mode': mode,
                           'error': e})
             LOG.error(error_msg)
+
+            # NOTE(sbaker): Some systems such as HPE Gen9 do not support
+            # getting or setting the boot mode. When setting failed and the
+            # mode attribute is missing from the boot field, raising
+            # UnsupportedDriverExtension will allow the deploy to continue.
+            if system.boot.get('mode') is None:
+                LOG.info(_('Attempt to set boot mode on node %(node)s '
+                           'failed to set boot mode as the node does not '
+                           'appear to support overriding the boot mode. '
+                           'Possibly partial Redfish implementation?'),
+                         {'node': task.node.uuid})
+                raise exception.UnsupportedDriverExtension(
+                    driver=task.node.driver, extension='set_boot_mode')
             raise exception.RedfishError(error=error_msg)
 
     def get_boot_mode(self, task):
@@ -1148,3 +1166,22 @@ class RedfishManagement(base.ManagementInterface):
         self._reset_keys(task, sushy.SECURE_BOOT_RESET_KEYS_DELETE_ALL)
         LOG.info('Secure boot keys have been removed from node %s',
                  task.node.uuid)
+
+    def get_mac_addresses(self, task):
+        """Get MAC address information for the node.
+
+        :param task: A TaskManager instance containing the node to act on.
+        :raises: RedfishConnectionError when it fails to connect to Redfish
+        :raises: RedfishError on an error from the Sushy library
+        :returns: a dictionary containing MAC addresses of enabled interfaces
+                  in a {'mac': 'state'} format
+        """
+        try:
+            system = redfish_utils.get_system(task.node)
+            return redfish_utils.get_enabled_macs(task, system)
+        except sushy.exceptions.SushyError as exc:
+            msg = (_('Failed to get network interface information on node '
+                     '%(node)s: %(exc)s')
+                   % {'node': task.node.uuid, 'exc': exc})
+            LOG.error(msg)
+            raise exception.RedfishError(error=msg)
