@@ -10,7 +10,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from oslo_context import context as oslo_context
 
 from ironic.common import context
@@ -22,18 +23,18 @@ class RequestContextTestCase(tests_base.TestCase):
         super(RequestContextTestCase, self).setUp()
         self.context_dict = {
             'auth_token': 'auth_token1',
-            "user": "user1",
-            "tenant": "tenant1",
+            "user_id": "user1",
+            "project_id": "project1",
             "project_name": "somename",
             'is_admin': True,
             'read_only': True,
             'show_deleted': True,
             'request_id': 'id1',
             "is_public_api": True,
-            "domain": "domain_id2",
-            "user_domain": "domain_id3",
+            "domain_id": "domain_id2",
+            "user_domain_id": "domain_id3",
             "user_domain_name": "TreeDomain",
-            "project_domain": "domain_id4",
+            "project_domain_id": "domain_id4",
             "roles": None,
             "overwrite": True
         }
@@ -44,14 +45,6 @@ class RequestContextTestCase(tests_base.TestCase):
         context_mock.assert_called_once_with(mock.ANY)
         self.assertFalse(test_context.is_public_api)
 
-    def test_from_dict(self):
-        test_context = context.RequestContext.from_dict(
-            {'project_name': 'demo', 'is_public_api': True,
-             'domain_id': 'meow'})
-        self.assertEqual('demo', test_context.project_name)
-        self.assertEqual('meow', test_context.user_domain)
-        self.assertTrue(test_context.is_public_api)
-
     def test_to_policy_values(self):
         ctx = context.RequestContext(**self.context_dict)
         ctx_dict = ctx.to_policy_values()
