@@ -156,12 +156,14 @@ SUSHY_SPEC = (
     'VIRTUAL_MEDIA_CD',
     'VIRTUAL_MEDIA_FLOPPY',
     'VIRTUAL_MEDIA_USBSTICK',
+    'APPLY_TIME_IMMEDIATE',
     'APPLY_TIME_ON_RESET',
     'TASK_STATE_COMPLETED',
     'HEALTH_OK',
     'HEALTH_WARNING',
     'SECURE_BOOT_RESET_KEYS_TO_DEFAULT',
     'SECURE_BOOT_RESET_KEYS_DELETE_ALL',
+    'VOLUME_TYPE_RAW_DEVICE'
 )
 
 SUSHY_AUTH_SPEC = (
